@@ -1,7 +1,8 @@
-import urllib
-import sys
+import requests
+import os
 from functools import wraps
 import inspect
+import sys
 # Pandas became an optional dependency, but we still want to track it
 try:
     import pandas
@@ -10,25 +11,21 @@ except ImportError:
     _PANDAS_FOUND = False
 import csv
 
-# Avoid compability issues
-if sys.version_info.major == 3 and sys.version_info.minor == 6:
-    from json import loads
-else:
-    from simplejson import loads
-
 
 class AlphaVantage(object):
     """ Base class where the decorators and base function for the other
     classes of this python wrapper will inherit from.
     """
-    _ALPHA_VANTAGE_API_URL = "http://www.alphavantage.co/query?"
+    _ALPHA_VANTAGE_API_URL = "https://www.alphavantage.co/query?"
     _ALPHA_VANTAGE_MATH_MAP = ['SMA', 'EMA', 'WMA', 'DEMA', 'TEMA', 'TRIMA',
                                'T3', 'KAMA', 'MAMA']
     _ALPHA_VANTAGE_DIGITAL_CURRENCY_LIST = \
         "https://www.alphavantage.co/digital_currency_list/"
 
-    def __init__(self, key=None, retries=5, output_format='json',
-                 treat_info_as_error=True, indexing_type='date'):
+    _RAPIDAPI_URL = "https://alpha-vantage.p.rapidapi.com/query?"
+
+    def __init__(self, key=None, output_format='json',
+                 treat_info_as_error=True, indexing_type='date', proxy=None, rapidapi=False):
         """ Initialize the class
 
         Keyword Arguments:
@@ -40,42 +37,40 @@ class AlphaVantage(object):
             indexing_type: Either 'date' to use the default date string given
             by the alpha vantage api call or 'integer' if you just want an
             integer indexing on your dataframe. Only valid, when the
-            output_format is 'pandas'.
+            output_format is 'pandas'
+            proxy: Dictionary mapping protocol or protocol and hostname to 
+            the URL of the proxy.
+            rapidapi: Boolean describing whether or not the API key is 
+            through the RapidAPI platform or not
         """
         if key is None:
-            raise ValueError(
-                'Get a free key from the alphavantage website:'
-                ' https://www.alphavantage.co/support/#api-key')
+            key = os.getenv('ALPHAVANTAGE_API_KEY')
+        if not key or not isinstance(key, str):
+            raise ValueError('The AlphaVantage API key must be provided '
+                             'either through the key parameter or '
+                             'through the environment variable '
+                             'ALPHAVANTAGE_API_KEY. Get a free key '
+                             'from the alphavantage website: '
+                             'https://www.alphavantage.co/support/#api-key')
+        self.headers = {}
+        if rapidapi:
+            self.headers = {
+                'x-rapidapi-host': "alpha-vantage.p.rapidapi.com",
+                'x-rapidapi-key': key
+            }
+        self.rapidapi = rapidapi
         self.key = key
-        self.retries = retries
         self.output_format = output_format
-        if self.output_format is 'pandas' and not _PANDAS_FOUND:
+        if self.output_format == 'pandas' and not _PANDAS_FOUND:
             raise ValueError("The pandas library was not found, therefore can "
                              "not be used as an output format, please install "
                              "manually")
         self.treat_info_as_error = treat_info_as_error
         # Not all the calls accept a data type appended at the end, this
-        # variable will be overriden by those functions not needing it.
+        # variable will be overridden by those functions not needing it.
         self._append_type = True
         self.indexing_type = indexing_type
-
-    def _retry(func):
-        """ Decorator for retrying api calls (in case of errors from the api
-        side in bringing the data)
-
-        Keyword Arguments:
-            func:  The function to be retried
-        """
-        @wraps(func)
-        def _retry_wrapper(self, *args, **kwargs):
-            error_message = ""
-            for retry in range(self.retries + 1):
-                try:
-                    return func(self, *args, **kwargs)
-                except ValueError as err:
-                    error_message = str(err)
-            raise ValueError(str(error_message))
-        return _retry_wrapper
+        self.proxy = proxy or {}
 
     @classmethod
     def _call_api_on_func(cls, func):
@@ -88,7 +83,11 @@ class AlphaVantage(object):
         """
 
         # Argument Handling
-        argspec = inspect.getargspec(func)
+        if sys.version_info[0] < 3:
+            # Deprecated since version 3.0
+            argspec = inspect.getargspec(func)
+        else:
+            argspec = inspect.getfullargspec(func)
         try:
             # Asumme most of the cases have a mixed between args and named
             # args
@@ -121,8 +120,8 @@ class AlphaVantage(object):
             # key for it and for its meta data.
             function_name, data_key, meta_data_key = func(
                 self, *args, **kwargs)
-            url = "{}function={}".format(AlphaVantage._ALPHA_VANTAGE_API_URL,
-                                         function_name)
+            base_url = AlphaVantage._RAPIDAPI_URL if self.rapidapi else AlphaVantage._ALPHA_VANTAGE_API_URL
+            url = "{}function={}".format(base_url, function_name)
             for idx, arg_name in enumerate(argspec.args[1:]):
                 try:
                     arg_value = args[idx]
@@ -143,7 +142,7 @@ class AlphaVantage(object):
                     url = '{}&{}={}'.format(url, arg_name, arg_value)
             # Allow the output format to be json or csv (supported by
             # alphavantage api). Pandas is simply json converted.
-            if 'json' or 'csv' in self.output_format.lower():
+            if 'json' in self.output_format.lower() or 'csv' in self.output_format.lower():
                 oformat = self.output_format.lower()
             elif 'pandas' in self.output_format.lower():
                 oformat = 'json'
@@ -151,10 +150,12 @@ class AlphaVantage(object):
                 raise ValueError("Output format: {} not recognized, only json,"
                                  "pandas and csv are supported".format(
                                      self.output_format.lower()))
+            apikey_parameter = "" if self.rapidapi else "&apikey={}".format(
+                self.key)
             if self._append_type:
-                url = '{}&apikey={}&datatype={}'.format(url, self.key, oformat)
+                url = '{}{}&datatype={}'.format(url, apikey_parameter, oformat)
             else:
-                url = '{}&apikey={}'.format(url, self.key)
+                url = '{}{}'.format(url, apikey_parameter)
             return self._handle_api_call(url), data_key, meta_data_key
         return _call_wrapper
 
@@ -174,6 +175,7 @@ class AlphaVantage(object):
             if 'json' in self.output_format.lower() or 'pandas' \
                     in self.output_format.lower():
                 data = call_response[data_key]
+
                 if meta_data_key is not None:
                     meta_data = call_response[meta_data_key]
                 else:
@@ -198,14 +200,31 @@ class AlphaVantage(object):
                         data_pandas = pandas.DataFrame(data_array, columns=[
                             k for k, _ in data[0].items()])
                     else:
-                        data_pandas = pandas.DataFrame.from_dict(data,
-                                                                 orient='index',
-                                                                 dtype=float)
-                    data_pandas.index.name = 'date'
+                        try:
+                            data_pandas = pandas.DataFrame.from_dict(data,
+                                                                     orient='index',
+                                                                     dtype='float')
+                        # This is for Global quotes or any other new Alpha Vantage
+                        # data that is added.
+                        # It will have to be updated so that we can get exactly
+                        # The dataframes we want moving forward
+                        except ValueError:
+                            data = {data_key: data}
+                            data_pandas = pandas.DataFrame.from_dict(data,
+                                                                     orient='index',
+                                                                     dtype='object')
+                            return data_pandas, meta_data
+
                     if 'integer' in self.indexing_type:
                         # Set Date as an actual column so a new numerical index
                         # will be created, but only when specified by the user.
                         data_pandas.reset_index(level=0, inplace=True)
+                        data_pandas.index.name = 'index'
+                    else:
+                        data_pandas.index.name = 'date'
+                        # convert to pandas._libs.tslibs.timestamps.Timestamp
+                        data_pandas.index = pandas.to_datetime(
+                            data_pandas.index)
                     return data_pandas, meta_data
             elif 'csv' in self.output_format.lower():
                 return call_response, None
@@ -214,9 +233,18 @@ class AlphaVantage(object):
                     self.output_format))
         return _format_wrapper
 
+    def set_proxy(self, proxy=None):
+        """ Set a new proxy configuration
+
+        Keyword Arguments:
+            proxy: Dictionary mapping protocol or protocol and hostname to 
+            the URL of the proxy.
+        """
+        self.proxy = proxy or {}
+
     def map_to_matype(self, matype):
         """ Convert to the alpha vantage math type integer. It returns an
-        integer correspondant to the type of math to apply to a function. It
+        integer correspondent to the type of math to apply to a function. It
         raises ValueError if an integer greater than the supported math types
         is given.
 
@@ -243,7 +271,6 @@ class AlphaVantage(object):
             value = AlphaVantage._ALPHA_VANTAGE_MATH_MAP.index(matype)
         return value
 
-    @_retry
     def _handle_api_call(self, url):
         """ Handle the return call from the  api and return a data and meta_data
         object. It raises a ValueError on problems
@@ -254,15 +281,10 @@ class AlphaVantage(object):
             meta_data_key:  The key for getting the meta data information out
             of the json object
         """
-        # In order to keep supporting python 2.7, we have to do this.
-        if sys.version_info.major == 3:
-            response = urllib.request.urlopen(url)
-        else:
-            response = urllib.urlopen(url)
-        url_response = response.read()
+        response = requests.get(url, proxies=self.proxy, headers=self.headers)
         if 'json' in self.output_format.lower() or 'pandas' in \
                 self.output_format.lower():
-            json_response = loads(url_response)
+            json_response = response.json()
             if not json_response:
                 raise ValueError(
                     'Error getting data from the api, no return was given.')
@@ -270,9 +292,11 @@ class AlphaVantage(object):
                 raise ValueError(json_response["Error Message"])
             elif "Information" in json_response and self.treat_info_as_error:
                 raise ValueError(json_response["Information"])
+            elif "Note" in json_response and self.treat_info_as_error:
+                raise ValueError(json_response["Note"])
             return json_response
         else:
-            csv_response = csv.reader(url_response.splitlines())
+            csv_response = csv.reader(response.text.splitlines())
             if not csv_response:
                 raise ValueError(
                     'Error getting data from the api, no return was given.')
