@@ -143,7 +143,7 @@ class TechIndicators(av):
 
     @av._output_format
     @av._call_api_on_func
-    def get_mama(self, symbol, interval='daily', time_period=20, series_type='close',
+    def get_mama(self, symbol, interval='daily', series_type='close',
                  fastlimit=None, slowlimit=None):
         """ Return MESA adaptative moving average time series in two json
         objects as data and meta_data. It raises ValueError when problems arise
@@ -153,7 +153,6 @@ class TechIndicators(av):
             interval:  time interval between two conscutive values,
                 supported values are '1min', '5min', '15min', '30min', '60min', 'daily',
                 'weekly', 'monthly' (default 'daily')
-            time_period:  How many data points to average (default 20)
             series_type:  The desired price type in the time series. Four types
                 are supported: 'close', 'open', 'high', 'low' (default 'close')
             fastlimit:  Positive floats for the fast limit are accepted
@@ -164,6 +163,20 @@ class TechIndicators(av):
         _FUNCTION_KEY = "MAMA"
         return _FUNCTION_KEY, 'Technical Analysis: MAMA', 'Meta Data'
 
+    @av._output_format
+    @av._call_api_on_func
+    def get_vwap(self, symbol, interval='5min'):
+        """ Returns the volume weighted average price (VWAP) for intraday time series.
+
+        Keyword Arguments:
+            symbol:  the symbol for the equity we want to get its data
+            interval:  time interval between two conscutive values,
+                supported values are '1min', '5min', '15min', '30min', '60min' 
+                (default 5min)
+        """
+        _FUNCTION_KEY = "VWAP"
+        return _FUNCTION_KEY, 'Technical Analysis: VWAP', 'Meta Data'
+
     @av._output_format
     @av._call_api_on_func
     def get_t3(self, symbol, interval='daily', time_period=20, series_type='close'):
@@ -676,7 +689,7 @@ class TechIndicators(av):
 
     @av._output_format
     @av._call_api_on_func
-    def get_ultsoc(self, symbol, interval='daily', timeperiod1=None,
+    def get_ultosc(self, symbol, interval='daily', timeperiod1=None,
                    timeperiod2=None, timeperiod3=None):
         """ Return the ultimate oscillaror values in two json objects as
         data and meta_data. It raises ValueError when problems arise
